import asyncio
import json
from langchain_openai import ChatOpenAI
from .chat import info
from .assistant import Runnable
from langchain_core.messages import (
    HumanMessage,
    AIMessage,
    AIMessageChunk,
    SystemMessage,
)
from langchain_core.agents import AgentFinish


async def run_agent(agent, inputs: list):
    if agent.agent_execution_mode == "FuncCall":
        exist_ids = set()
        content_list = []
        async for event in agent.astream_events(inputs, version="v2"):
            kind = event["event"]
            if kind == "on_chain_end":
                if "data" in event:
                    if (
                        "output" in event["data"]
                        and event["data"]["output"] == "end"
                        and "input" in event["data"]
                        and isinstance(event["data"]["input"], list)
                    ):
                        input_list = event["data"]["input"]
                        for msg in input_list:
                            if isinstance(msg, HumanMessage) or isinstance(
                                msg, SystemMessage
                            ):
                                content_list = []
                            if isinstance(msg, AIMessage) and not isinstance(
                                msg, AIMessageChunk
                            ):
                                content = msg.content
                                if content:
                                    content_list.append(content)
            elif kind == "on_chat_model_stream":
                content = event["data"]["chunk"].content
                if content:
                    info("assistant", content)
            elif kind == "on_chain_start":
                data = event["data"]
                if "input" in data:
                    input_list = (
                        data["input"]
                        if isinstance(data["input"], list)
                        else [data["input"]]
                    )
                    if len(input_list) == 0:
                        continue
                    msg = input_list[-1]
                    if isinstance(msg, AIMessage) and not isinstance(
                        msg, AIMessageChunk
                    ):
                        if "tool_calls" in msg.additional_kwargs:
                            tool_calls = msg.additional_kwargs["tool_calls"]
                            for t in tool_calls:
                                if t["id"] in exist_ids:
                                    continue
                                exist_ids.add(t["id"])
                                tool = t["function"]["name"]
                                info("assistant", f"\n[调用工具:{tool}]\n\n")

        return "\n".join(content_list)
    else:
        assert agent.agent_execution_mode == "ReAct"
        inputs_msg = {"input": inputs[-1].content, "chat_history": inputs[:-1]}
        use_tools = []
        async for event in agent.astream_events(inputs_msg, version="v2"):
            kind = event["event"]
            result = None
            if kind == "on_chain_end":
                if "data" in event:
                    if "output" in event["data"]:
                        output = event["data"]["output"]
                        if "agent_outcome" in output and "input" in output:
                            outcome = output["agent_outcome"]
                            if isinstance(outcome, AgentFinish):
                                result = outcome.return_values["output"]
            elif kind == "on_tool_start":
                use_tools.append(event["name"])
                info("assistant", f"\n[调用工具:{use_tools}]\n\n")
        return result


if __name__ == "__main__":
    from langchain_experimental.tools import PythonREPLTool

    llm_file = r"C:\Users\aiqqq\AppData\Roaming\pycoze\JsonStorage\llm.json"
    with open(llm_file, "r", encoding="utf-8") as f:
        cfg = json.load(f)
        chat = ChatOpenAI(
            api_key=cfg["apiKey"],
            base_url=cfg["baseURL"],
            model=cfg["model"],
            temperature=0,
        )
    python_tool = PythonREPLTool()
    agent = Runnable(
        agent_execution_mode="FuncCall",  # 'FuncCall' or 'ReAct'，大模型支持FuncCall的话就用FuncCall
        tools=[python_tool],
        llm=chat,
        assistant_message="请以女友的口吻回答，输出不小于100字，可以随便说点其他的",
    )

    inputs = [HumanMessage(content="计算根号7+根号88")]
    print(asyncio.run(run_agent(agent, inputs)))



import json
from langchain_openai import ChatOpenAI
from .agent import run_agent, Runnable, INPUT_MESSAGE, output
import asyncio
from langchain_core.messages import HumanMessage
from pycoze import utils
from pycoze.access.tool_for_bot import import_tools

params = utils.arg.read_params()
llm_file = params["appPath"] + "/JsonStorage/llm.json"


def load_role_setting(bot_setting_file: str):
    with open(bot_setting_file, "r", encoding="utf-8") as f:
        return json.load(f)


def load_tools(bot_setting_file: str):
    with open(bot_setting_file, "r", encoding="utf-8") as f:
        role_setting = json.load(f)

    tools = []
    for tool_id in role_setting["tools"]:
        tools.extend(import_tools(tool_id))
    return tools


def agent_chat(bot_setting_file, history):
    role_setting = load_role_setting(bot_setting_file)
    tools = load_tools(bot_setting_file)
    with open(llm_file, "r", encoding="utf-8") as f:
        cfg = json.load(f)
        chat = ChatOpenAI(
            api_key=cfg["apiKey"],
            base_url=cfg["baseURL"],
            model=cfg["model"],
            temperature=(
                role_setting["temperature"] * 2
                if cfg["model"].startswith("deepseek")
                else role_setting["temperature"]
            ),
            stop_sequences=[
                "tool▁calls▁end",
                "tool▁call▁end",
            ],  # 停用deepseek的工具调用标记，不然会虚构工具调用过程和结果
        )
    prompt = role_setting["prompt"]
    if cfg["model"].startswith("deepseek") and len(tools) > 0:
        prompt += """
如果需要调用工具，请使用以正确的json格式进行结尾（务必保证json格式正确，不要出现反斜杠未转义等问题）：
```json
{"name": 函数名, "parameters": 参数词典}
```
"""
    agent = Runnable(
        agent_execution_mode=(
            "ReAct" if cfg["model"] in ["command-r"] else "FuncCall"
        ),  # 'FuncCall' or 'ReAct'，大模型支持FuncCall的话就用FuncCall
        tools=tools,
        llm=chat,
        assistant_message=prompt,
    )
    return asyncio.run(run_agent(agent, history))


def chat(bot_setting_file: str):
    history = []
    while True:
        input_text = input()
        if not input_text.startswith(INPUT_MESSAGE):
            raise ValueError("Invalid message")
        message = json.loads(input_text[len(INPUT_MESSAGE) :])
        history.append(HumanMessage(message["content"]))
        result = agent_chat(bot_setting_file, history)
        history.append(output("assistant", result))



from .openai_func_call_agent import create_openai_func_call_agent_executor
from .react_agent import create_react_agent_executor


__all__ = [create_openai_func_call_agent_executor, create_react_agent_executor]
